# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import six

from .common import PingTestData


# ping google.com -q -c 60:
#   - Debian 8.2 w/ iputils-ping 20121221-5+b2
#   - Debian 5.0.10 w/ iputils-ping 20071127-1+lenny1
PING_DEBIAN_SUCCESS = six.b("""PING google.com (216.58.196.238) 56(84) bytes of data.

--- google.com ping statistics ---
60 packets transmitted, 60 received, 0% packet loss, time 59153ms
rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
""")


DEBIAN_SUCCESS = PingTestData(
    PING_DEBIAN_SUCCESS,
    {
        "destination": "google.com",
        "packet_transmit": 60,
        "packet_receive": 60,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_rate": 0,
        "packet_duplicate_count": 0,
        "rtt_min": 61.425,
        "rtt_avg": 99.731,
        "rtt_max": 212.597,
        "rtt_mdev": 27.566,
    })

WINDOWS7SP1_SUCCESS = PingTestData(
    # ping google.com -n 10:
    """
Pinging google.com [216.58.196.238] with 32 bytes of data:
Reply from 216.58.196.238: bytes=32 time=87ms TTL=51
Reply from 216.58.196.238: bytes=32 time=97ms TTL=51
Reply from 216.58.196.238: bytes=32 time=56ms TTL=51
Reply from 216.58.196.238: bytes=32 time=95ms TTL=51
Reply from 216.58.196.238: bytes=32 time=194ms TTL=51
Reply from 216.58.196.238: bytes=32 time=98ms TTL=51
Reply from 216.58.196.238: bytes=32 time=93ms TTL=51
Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
Reply from 216.58.196.238: bytes=32 time=96ms TTL=51
Reply from 216.58.196.238: bytes=32 time=165ms TTL=51

Ping statistics for 216.58.196.238:
    Packets: Sent = 10, Received = 10, Lost = 0 (0% loss),
Approximate round trip times in milli-seconds:
    Minimum = 56ms, Maximum = 194ms, Average = 107ms
""",
    {
        "destination": "216.58.196.238",
        "packet_transmit": 10,
        "packet_receive": 10,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": 56,
        "rtt_avg": 107,
        "rtt_max": 194,
        "rtt_mdev": None,
    })



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import json
import sys

import pingparsing


def main():
    parser = pingparsing.PingParsing()
    parser.parse("""PING google.com (216.58.196.238) 56(84) bytes of data.

--- google.com ping statistics ---
60 packets transmitted, 60 received, 0% packet loss, time 59153ms
rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
""")

    print(json.dumps(parser.as_dict(), indent=4))

    return 0


if __name__ == "__main__":
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import abc

import six


@six.add_metaclass(abc.ABCMeta)
class PingParserInterface(object):

    @abc.abstractproperty
    def packet_transmit(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def packet_receive(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def packet_loss_rate(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def packet_loss_count(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def rtt_min(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def rtt_avg(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def rtt_max(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def rtt_mdev(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def packet_duplicate_rate(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def packet_duplicate_count(self):  # pragma: no cover
        pass



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

import pytest
from pingparsing import PingResult

from .common import ping_parser
from .data import DEBIAN_SUCCESS


class Test_PingResult(object):

    @pytest.mark.parametrize(["pingresult", "expected"], [
        [
            PingResult(DEBIAN_SUCCESS.value, "", 0),
            DEBIAN_SUCCESS.expected
        ]
    ])
    def test_normal_pingresult(self, ping_parser, pingresult, expected):
        ping_parser.parse(pingresult.stdout)
        assert ping_parser.as_dict() == expected

        ping_parser.parse(pingresult)
        assert ping_parser.as_dict() == expected



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import abc
import re

import pyparsing as pp
import typepy

from ._common import _to_unicode
from ._interface import PingParserInterface
from ._logger import logger
from .error import EmptyPingStatisticsError, PingStatisticsHeaderNotFoundError


class PingParser(PingParserInterface):

    def __init__(self):
        self.ping_option = ""

        self.__initialize_parse_result()

    @property
    def destination(self):
        return self._destination

    @property
    def packet_transmit(self):
        return self._packet_transmit

    @property
    def packet_receive(self):
        return self._packet_receive

    @property
    def packet_loss_rate(self):
        try:
            return (1.0 - (self.packet_receive / self.packet_transmit)) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def packet_loss_count(self):
        try:
            return self.packet_transmit - self.packet_receive
        except TypeError:
            return None

    @property
    def rtt_min(self):
        return self._rtt_min

    @property
    def rtt_avg(self):
        return self._rtt_avg

    @property
    def rtt_max(self):
        return self._rtt_max

    @property
    def rtt_mdev(self):
        return self._rtt_mdev

    @property
    def packet_duplicate_rate(self):
        try:
            return (self.packet_duplicate_count / self.packet_receive) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def packet_duplicate_count(self):
        return self._duplicates

    @abc.abstractproperty
    def _parser_name(self):  # pragma: no cover
        pass

    @abc.abstractproperty
    def _stats_headline_pattern(self):  # pragma: no cover
        pass

    def _preprocess_parse(self, line_list):
        logger.debug(
            "parsing as {:s} ping result format".format(self._parser_name))

        self.__initialize_parse_result()

        headline_idx = self.__find_stats_headline_idx(
            line_list, re.compile(self._stats_headline_pattern))
        body_line_list = line_list[headline_idx + 1:]
        self.__validate_stats_body(body_line_list)

        packet_info_line = body_line_list[0]

        return (line_list[headline_idx], packet_info_line, body_line_list)

    def _parse_destination(self, headline):
        return headline.lstrip("--- ").rstrip(" ping statistics ---")

    def __find_stats_headline_idx(self, line_list, re_stats_header):
        for i, line in enumerate(line_list):
            if re_stats_header.search(line):
                break
        else:
            raise PingStatisticsHeaderNotFoundError(
                "ping statistics not found")

        return i

    def __validate_stats_body(self, body_line_list):
        if typepy.is_empty_sequence(body_line_list):
            raise EmptyPingStatisticsError("ping statistics is empty")

    def __initialize_parse_result(self):
        self._destination = None
        self._packet_transmit = None
        self._packet_receive = None
        self._rtt_min = None
        self._rtt_avg = None
        self._rtt_max = None
        self._rtt_mdev = None
        self._duplicates = None

    def _parse_duplicate(self, line):
        packet_pattern = (
            pp.SkipTo(pp.Word("+" + pp.nums) + pp.Literal("duplicates,")) +
            pp.Word("+" + pp.nums) +
            pp.Literal("duplicates,")
        )
        try:
            duplicate_parse_list = packet_pattern.parseString(_to_unicode(line))
        except pp.ParseException:
            return 0

        return int(duplicate_parse_list[-2].strip("+"))


class NullPingParser(PingParser):

    @property
    def _parser_name(self):
        return "null"

    @property
    def _stats_headline_pattern(self):
        return ""

    def parse(self, ping_message):  # pragma: no cover
        pass

    def _preprocess_parse(self, line_list):  # pragma: no cover
        pass


class LinuxPingParser(PingParser):

    @property
    def _parser_name(self):
        return "Linux"

    @property
    def _stats_headline_pattern(self):
        return "--- .* ping statistics ---"

    def parse(self, ping_message):
        headline, packet_info_line, body_line_list = self._preprocess_parse(
            line_list=ping_message)
        packet_pattern = (
            pp.Word(pp.nums) +
            pp.Literal("packets transmitted,") +
            pp.Word(pp.nums) +
            pp.Literal("received,")
        )

        self._destination = self._parse_destination(headline)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        self._packet_transmit = int(parse_list[0])
        self._packet_receive = int(parse_list[2])
        self._duplicates = self._parse_duplicate(packet_info_line)

        try:
            rtt_line = body_line_list[1]
        except IndexError:
            return

        if typepy.is_null_string(rtt_line):
            return

        rtt_pattern = (
            pp.Literal("rtt min/avg/max/mdev =") +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") +
            pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        self._rtt_min = float(parse_list[1])
        self._rtt_avg = float(parse_list[3])
        self._rtt_max = float(parse_list[5])
        self._rtt_mdev = float(parse_list[7])


class WindowsPingParser(PingParser):

    @property
    def _parser_name(self):
        return "Windows"

    @property
    def _stats_headline_pattern(self):
        return "^Ping statistics for "

    def parse(self, ping_message):
        headline, packet_info_line, body_line_list = self._preprocess_parse(
            line_list=ping_message)
        packet_pattern = (
            pp.Literal("Packets: Sent = ") +
            pp.Word(pp.nums) +
            pp.Literal(", Received = ") +
            pp.Word(pp.nums)
        )

        self._destination = self._parse_destination(headline)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        self._packet_transmit = int(parse_list[1])
        self._packet_receive = int(parse_list[3])

        try:
            rtt_line = body_line_list[2].strip()
        except IndexError:
            return

        if typepy.is_null_string(rtt_line):
            return

        rtt_pattern = (
            pp.Literal("Minimum = ") +
            pp.Word(pp.nums) +
            pp.Literal("ms, Maximum = ") +
            pp.Word(pp.nums) +
            pp.Literal("ms, Average = ") +
            pp.Word(pp.nums)
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        self._rtt_min = float(parse_list[1])
        self._rtt_avg = float(parse_list[5])
        self._rtt_max = float(parse_list[3])

    def _parse_destination(self, headline):
        return headline.lstrip("Ping statistics for ").rstrip(":")


class MacOsPingParser(PingParser):

    @property
    def _parser_name(self):
        return "macOS"

    @property
    def _stats_headline_pattern(self):
        return "--- .* ping statistics ---"

    def parse(self, ping_message):
        headline, packet_info_line, body_line_list = self._preprocess_parse(
            line_list=ping_message)
        packet_pattern = (
            pp.Word(pp.nums) +
            pp.Literal("packets transmitted,") +
            pp.Word(pp.nums) +
            pp.Literal("packets received,")
        )

        self._destination = self._parse_destination(headline)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        self._packet_transmit = int(parse_list[0])
        self._packet_receive = int(parse_list[2])
        self._duplicates = self._parse_duplicate(packet_info_line)

        try:
            rtt_line = body_line_list[1]
        except IndexError:
            return

        if typepy.is_null_string(rtt_line):
            return

        rtt_pattern = (
            pp.Literal("round-trip min/avg/max/stddev =") +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") +
            pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        self._rtt_min = float(parse_list[1])
        self._rtt_avg = float(parse_list[3])
        self._rtt_max = float(parse_list[5])
        self._rtt_mdev = float(parse_list[7])


class AlpineLinuxPingParser(LinuxPingParser):

    @property
    def _parser_name(self):
        return "AlpineLinux"

    def parse(self, ping_message):
        headline, packet_info_line, body_line_list = self._preprocess_parse(
            line_list=ping_message)
        packet_pattern = (
            pp.Word(pp.nums) +
            pp.Literal("packets transmitted,") +
            pp.Word(pp.nums) +
            pp.Literal("packets received,")
        )

        self._destination = self._parse_destination(headline)

        parse_list = packet_pattern.parseString(_to_unicode(packet_info_line))
        self._packet_transmit = int(parse_list[0])
        self._packet_receive = int(parse_list[2])
        self._duplicates = self._parse_duplicate(packet_info_line)

        try:
            rtt_line = body_line_list[1]
        except IndexError:
            return

        if typepy.is_null_string(rtt_line):
            return

        rtt_pattern = (
            pp.Literal("round-trip min/avg/max =") +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") + "/" +
            pp.Word(pp.nums + ".") +
            pp.Word(pp.nums + "ms")
        )
        parse_list = rtt_pattern.parseString(_to_unicode(rtt_line))
        self._rtt_min = float(parse_list[1])
        self._rtt_avg = float(parse_list[3])
        self._rtt_max = float(parse_list[5])

    def _parse_duplicate(self, line):
        packet_pattern = (
            pp.SkipTo(pp.Word(pp.nums) + pp.Literal("duplicates,")) +
            pp.Word(pp.nums) +
            pp.Literal("duplicates,")
        )
        try:
            duplicate_parse_list = packet_pattern.parseString(
                _to_unicode(line))
        except pp.ParseException:
            return 0

        return int(duplicate_parse_list[-2])



#!/usr/bin/env python
# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function

import argparse
import json
import multiprocessing
import os
import sys

import logbook
import pingparsing


DEFAULT_COUNT = 10
QUIET_LOG_LEVEL = logbook.NOTSET


def parse_option():
    parser = argparse.ArgumentParser(
        epilog="Issue tracker: https://github.com/thombashi/pingparsing/issues")

    if is_use_stdin():
        parser.add_argument(
            "destination_or_file", nargs="+",
            help="")

    parser.add_argument(
        "--max-workers", type=int,
        help="""a number of threads for when multiple destination/file
        specified. defaults to equals to two times number of cores.
        """)
    parser.add_argument(
        "--indent", type=int, default=4,
        help="""JSON output will be pretty-printed with the indent level.
        (default= %(default)s)
        """)

    loglevel_dest = "log_level"
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "--debug", dest=loglevel_dest, action="store_const",
        const=logbook.DEBUG, default=logbook.INFO,
        help="for debug print.")
    group.add_argument(
        "--quiet", dest=loglevel_dest, action="store_const",
        const=logbook.NOTSET, default=logbook.INFO,
        help="suppress execution log messages.")

    group = parser.add_argument_group("Ping Options")
    group.add_argument(
        "-c", "--count", type=int,
        help="""stop after sending the count.
        see also ping(8) [-c count] option description.
        """)
    group.add_argument(
        "-w", "--deadline", type=float,
        help="""timeout in seconds.
        see also ping(8) [-w deadline] option description.
        """)
    group.add_argument(
        "-I", "--interface", dest="interface",
        help="network interface")

    return parser.parse_args()


def initialize_log_handler(log_level):
    debug_format_str = (
        "[{record.level_name}] {record.channel} {record.func_name} "
        "({record.lineno}): {record.message}")
    if log_level == logbook.DEBUG:
        info_format_str = debug_format_str
    else:
        info_format_str = (
            "[{record.level_name}] {record.channel}: {record.message}")

    logbook.StderrHandler(
        level=logbook.DEBUG, format_string=debug_format_str
    ).push_application()
    logbook.StderrHandler(
        level=logbook.INFO, format_string=info_format_str
    ).push_application()


def is_use_stdin():
    return sys.stdin.isatty() or len(sys.argv) > 1


def parse_ping(logger, dest_or_file, interface, count, deadline):
    if os.path.isfile(dest_or_file):
        with open(dest_or_file) as f:
            ping_result_text = f.read()
    else:
        transmitter = pingparsing.PingTransmitter()
        transmitter.destination_host = dest_or_file
        transmitter.interface = interface
        transmitter.count = count
        transmitter.deadline = deadline
        result = transmitter.ping()
        ping_result_text = result.stdout
        if result.returncode != 0:
            logger.error(result.stderr)

    ping_parser = pingparsing.PingParsing()
    ping_parser.parse(ping_result_text)

    return (dest_or_file, ping_parser.as_dict())


def get_ping_param(options):
    count = options.count
    deadline = options.deadline

    if not options.count and not options.deadline:
        count = DEFAULT_COUNT

    return (count, deadline)


def main():
    options = parse_option()

    initialize_log_handler(options.log_level)

    logger = logbook.Logger("pingparsing cli")
    logger.level = options.log_level
    pingparsing.set_log_level(options.log_level)

    output = {}
    if is_use_stdin():
        from concurrent import futures

        pingparsing.set_log_level(options.log_level)

        max_workers = (multiprocessing.cpu_count() * 2
                       if options.max_workers is None else options.max_workers)
        count, deadline = get_ping_param(options)
        logger.debug("max-workers={}, count={}, deadline={}".format(
            max_workers, count, deadline))

        try:
            with futures.ProcessPoolExecutor(max_workers) as executor:
                future_list = []
                for dest_or_file in options.destination_or_file:
                    logger.debug("start {}".format(dest_or_file))
                    future_list.append(executor.submit(
                        parse_ping, logger, dest_or_file, options.interface,
                        count, deadline))

                for future in futures.as_completed(future_list):
                    key, ping_data = future.result()
                    output[key] = ping_data
        finally:
            logger.debug("shutdown ProcessPoolExecutor")
            executor.shutdown()
    else:
        ping_result_text = sys.stdin.read()
        ping_parser = pingparsing.PingParsing()
        ping_parser.parse(ping_result_text)
        output = ping_parser.as_dict()

    if options.indent <= 0:
        print(json.dumps(output))
    else:
        print(json.dumps(output, indent=options.indent))

    return 0


if __name__ == "__main__":
    sys.exit(main())



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import, division

import pyparsing as pp
import typepy

from ._common import _to_unicode
from ._interface import PingParserInterface
from ._logger import logger
from ._parser import (
    AlpineLinuxPingParser, LinuxPingParser, NullPingParser, MacOsPingParser, WindowsPingParser)
from .error import PingStatisticsHeaderNotFoundError


class PingParsing(PingParserInterface):
    """
    Parser class to parsing ping command output.
    """

    def __init__(self):
        self.ping_option = ""

        self.__parser = NullPingParser()

    @property
    def parser_name(self):
        return self.__parser._parser_name

    @property
    def destination(self):
        """
        :return: The ping destination.
        :rtype: str
        """

        return self.__parser.destination

    @property
    def packet_transmit(self):
        """
        :return: Number of packets transmitted.
        :rtype: int
        """

        return self.__parser.packet_transmit

    @property
    def packet_receive(self):
        """
        :return: Number of packets received.
        :rtype: int
        """

        return self.__parser.packet_receive

    @property
    def packet_loss_rate(self):
        """
        :return:
            Percentage of packet loss ``[%]``.
            ``None`` if the value is not a number.
        :rtype: float
        """

        try:
            return (1.0 - (self.packet_receive / self.packet_transmit)) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def packet_loss(self):
        # mark as delete
        return self.packet_loss_rate

    @property
    def packet_loss_count(self):
        """
        :return: Packet loss count. ``None`` if the value is not a number.
        :rtype: int
        """

        try:
            return self.packet_transmit - self.packet_receive
        except TypeError:
            return None

    @property
    def rtt_min(self):
        """
        :return:
            Minimum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__parser.rtt_min

    @property
    def rtt_avg(self):
        """
        :return:
            Average round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__parser.rtt_avg

    @property
    def rtt_max(self):
        """
        :return:
            Maximum round trip time of transmitted ICMP packets ``[msec]``.
        :rtype: float
        """

        return self.__parser.rtt_max

    @property
    def rtt_mdev(self):
        """
        :return:
            Standard deviation of transmitted ICMP packets. The attribute
            returns always ``None`` when parsing Windows ping result.
        :rtype: float
        """

        return self.__parser.rtt_mdev

    @property
    def packet_duplicate_rate(self):
        """
        :return:
            Percentage of duplicated packets ``[%]``.
            ``None`` if the value is not a number.
        :rtype: float
        """

        try:
            return (self.packet_duplicate_count / self.packet_receive) * 100
        except (TypeError, ZeroDivisionError, OverflowError):
            return None

    @property
    def packet_duplicate_count(self):
        """
        :return:
            Number of duplicated packet. The attribute
            returns always ``None`` when parsing Windows ping result.
        :rtype: int
        """

        return self.__parser.packet_duplicate_count

    @property
    def duplicates(self):
        # mark as delete
        return self.packet_duplicate_count

    def as_dict(self):
        """
        :return: Parsed result as a dictionary.
        :rtype: dict

        :Examples:
            >>> import pingparsing
            >>> parser = pingparsing.PingParsing()
            >>> parser.parse(ping_result)
            >>> parser.as_dict()
            {
                "destination": "google.com",
                "packet_transmit": 60,
                "packet_receive": 60,
                "packet_loss_rate": 0.0,
                "packet_loss_count": 0,
                "rtt_min": 61.425,
                "rtt_avg": 99.731,
                "rtt_max": 212.597,
                "rtt_mdev": 27.566,
                "packet_duplicate_rate": 0.0,
                "packet_duplicate_count": 0
            }
        """

        return {
            "destination": self.destination,
            "packet_transmit": self.packet_transmit,
            "packet_receive": self.packet_receive,
            "packet_loss_rate": self.packet_loss_rate,
            "packet_loss_count": self.packet_loss_count,
            "rtt_min": self.rtt_min,
            "rtt_avg": self.rtt_avg,
            "rtt_max": self.rtt_max,
            "rtt_mdev": self.rtt_mdev,
            "packet_duplicate_rate": self.packet_duplicate_rate,
            "packet_duplicate_count": self.packet_duplicate_count,
        }

    def as_tuple(self):
        """
        :return: Parsed result as a tuple.
        :rtype: collections.namedtuple

        :Examples:
            >>> import pingparsing
            >>> parser = pingparsing.PingParsing()
            >>> parser.parse(ping_result)
            >>> parser.as_tuple()
            PingResult(destination='google.com', packet_transmit=60, packet_receive=60, packet_loss_rate=0.0, packet_loss_count=0, rtt_min=61.425, rtt_avg=99.731, rtt_max=212.597, rtt_mdev=27.566, packet_duplicate_rate=0.0, packet_duplicate_count=0)
        """

        from collections import namedtuple

        ping_result = self.as_dict()

        return namedtuple("PingResult", ping_result.keys())(**ping_result)

    def parse(self, ping_message):
        """
        Parse ping command output.
        You can get parsing results by following attributes:

            - :py:attr:`.packet_transmit`
            - :py:attr:`.packet_receive`
            - :py:attr:`.packet_loss_rate`
            - :py:attr:`.packet_loss_count`
            - :py:attr:`.packet_duplicate_rate`
            - :py:attr:`.packet_duplicate_count`
            - :py:attr:`.rtt_min`
            - :py:attr:`.rtt_avg`
            - :py:attr:`.rtt_max`
            - :py:attr:`.rtt_mdev`

        Alternatively, you can get the results as:

            - a dictionary by :py:meth:`.as_dict`
            - a tuple by by :py:meth:`.as_tuple`

        :param ping_message: ping command output.
        :type ping_message: str or pingparsing.PingResult
        :return: Parsed result.

            .. seealso:: :py:meth:`.as_tuple`
        :rtype: collections.namedtuple
        """

        try:
            # accept PingResult instance as an input
            if typepy.is_not_null_string(ping_message.stdout):
                ping_message = ping_message.stdout
        except AttributeError:
            pass

        logger.debug("parsing ping result: {}".format(ping_message))

        self.__parser = NullPingParser()

        if typepy.is_null_string(ping_message):
            logger.debug("ping_message is empty")
            return self.as_tuple()

        line_list = _to_unicode(ping_message).splitlines()
        parser_class_list = (
            LinuxPingParser, WindowsPingParser, MacOsPingParser, AlpineLinuxPingParser)

        for parser_class in parser_class_list:
            self.__parser = parser_class()
            try:
                self.__parser.parse(line_list)
                return self.as_tuple()
            except (PingStatisticsHeaderNotFoundError, pp.ParseException):
                pass

        self.__parser = NullPingParser()

        return self.as_tuple()



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import absolute_import

from textwrap import dedent

import pytest
import six

from pingparsing import EmptyPingStatisticsError

from .common import PingTestData, ping_parser
from .data import DEBIAN_SUCCESS, WINDOWS7SP1_SUCCESS

PING_FEDORA_EMPTY_BODY = six.b("""\
PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

--- 192.168.0.1 ping statistics ---
""")
PING_WINDOWS_INVALID = """\
Pinging 192.168.207.100 with 32 bytes of data:
Request timed out.
Request timed out.
Request timed out.
Request timed out.

Ping statistics for 192.168.207.100:
"""

DEBIAN_UNREACHABLE_0 = PingTestData(
    dedent("""\
    PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.

    --- 192.168.207.100 ping statistics ---
    5 packets transmitted, 0 received, 100% packet loss, time 4009ms
    """),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 5,
        "packet_receive": 0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "packet_loss_count": 5,
        "packet_loss_rate": 100.0,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    })
DEBIAN_UNREACHABLE_1 = PingTestData(
    DEBIAN_UNREACHABLE_0.value + "\n", DEBIAN_UNREACHABLE_0.expected)
DEBIAN_UNREACHABLE_2 = PingTestData(
    DEBIAN_UNREACHABLE_1.value + "\n", DEBIAN_UNREACHABLE_0.expected)

FEDORA_DUP_LOSS = PingTestData(
    dedent("""\
    PING 192.168.0.1 (192.168.0.1) 56(84) bytes of data.

    --- 192.168.0.1 ping statistics ---
    1688 packets transmitted, 1553 received, +1 duplicates, 7% packet loss, time 2987ms
    rtt min/avg/max/mdev = 0.282/0.642/11.699/0.699 ms, pipe 2, ipg/ewma 1.770/0.782 ms
    """),
    {
        "destination": "192.168.0.1",
        "packet_transmit": 1688,
        "packet_receive": 1553,
        "packet_duplicate_count": 1,
        "packet_duplicate_rate": 0.0643915003219575,
        "packet_loss_count": 135,
        "packet_loss_rate": 7.997630331753558,
        "rtt_min": 0.282,
        "rtt_max": 11.699,
        "rtt_mdev": 0.699,
        "rtt_avg": 0.642,
    })
FEDORA_UNREACHABLE = PingTestData(
    dedent("""\
    PING 192.168.207.100 (192.168.207.100) 56(84) bytes of data.
    From 192.168.207.128 icmp_seq=1 Destination Host Unreachable
    From 192.168.207.128 icmp_seq=2 Destination Host Unreachable
    From 192.168.207.128 icmp_seq=3 Destination Host Unreachable
    From 192.168.207.128 icmp_seq=4 Destination Host Unreachable
    From 192.168.207.128 icmp_seq=5 Destination Host Unreachable

    --- 192.168.207.100 ping statistics ---
    5 packets transmitted, 0 received, +5 errors, 100% packet loss, time 4003ms
    """),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 5,
        "packet_receive": 0,
        "packet_loss_count": 5,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    })

MACOS_SUCCESS_0 = PingTestData(
    dedent("""\
    PING google.com (172.217.6.238): 56 data bytes
    64 bytes from 172.217.6.238: icmp_seq=0 ttl=53 time=20.482 ms
    64 bytes from 172.217.6.238: icmp_seq=1 ttl=53 time=32.550 ms
    64 bytes from 172.217.6.238: icmp_seq=2 ttl=53 time=32.013 ms
    64 bytes from 172.217.6.238: icmp_seq=3 ttl=53 time=28.498 ms
    64 bytes from 172.217.6.238: icmp_seq=4 ttl=53 time=46.093 ms

    --- google.com ping statistics ---
    5 packets transmitted, 5 packets received, 0.0% packet loss
    round-trip min/avg/max/stddev = 20.482/31.927/46.093/8.292 ms
    """),
    {
        "destination": "google.com",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 20.482,
        "rtt_avg": 31.927,
        "rtt_max": 46.093,
        "rtt_mdev": 8.292,
    })
MACOS_SUCCESS_1 = PingTestData(
    dedent("""\
    PING github.com (192.30.255.113): 56 data bytes

    --- github.com ping statistics ---
    10 packets transmitted, 10 packets received, 0.0% packet loss
    round-trip min/avg/max/stddev = 218.391/283.477/405.879/70.170 ms
    """),
    {
        "destination": "github.com",
        "packet_transmit": 10,
        "packet_receive": 10,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 218.391,
        "rtt_avg": 283.477,
        "rtt_max": 405.879,
        "rtt_mdev": 70.170,
    })
MACOS_UNREACHABLE_0 = PingTestData(
    dedent("""\
    PING twitter.com (59.24.3.173): 56 data bytes
    ^C
    --- twitter.com ping statistics ---
    59 packets transmitted, 0 packets received, 100.0% packet loss
    """),
    {
        "destination": "twitter.com",
        "packet_transmit": 59,
        "packet_receive": 0,
        "packet_loss_rate": 100.0,
        "packet_loss_count": 59,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
        "packet_duplicate_rate": None,
        "packet_duplicate_count": 0,
    })
MACOS_UNREACHABLE_1 = PingTestData(
    dedent("""\
    PING twitter.com (31.13.78.66): 56 data bytes

    --- twitter.com ping statistics ---
    10 packets transmitted, 0 packets received, 100.0% packet loss
    """),
    {
        "destination": "twitter.com",
        "packet_transmit": 10,
        "packet_receive": 0,
        "packet_loss_count": 10,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    })
MACOS_DUPLICATE_0 = PingTestData(
    dedent("""\
    PING duplicate.com (31.13.78.66): 56 data bytes

    --- duplicate.com ping statistics ---
    3 packets transmitted, 3 packets received, +3 duplicates, 0% packet loss

    round-trip min/avg/max/stddev = 0.311/1.091/2.186/0.662 ms
    """),
    {
        "destination": "duplicate.com",
        "packet_transmit": 3,
        "packet_receive": 3,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 3,
        "packet_duplicate_rate": 100,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    })

ALPINE_LINUX_SUCCESS = PingTestData(
   dedent("""\
   PING heise.de (193.99.144.80): 56 data bytes

    --- heise.de ping statistics ---
    5 packets transmitted, 5 packets received, 0% packet loss
    round-trip min/avg/max = 0.638/0.683/0.746 ms
    """),
    {
        "destination": "heise.de",
        "packet_transmit": 5,
        "packet_receive": 5,
        "packet_loss_count": 0,
        "packet_loss_rate": 0.0,
        "packet_duplicate_count": 0,
        "packet_duplicate_rate": 0,
        "rtt_min": 0.638,
        "rtt_avg": 0.683,
        "rtt_max": 0.746,
        "rtt_mdev": None,
    })
ALPINE_LINUX_DUP_LOSS = PingTestData(
    dedent("""\
    PING 192.168.2.106 (192.168.2.106): 56 data bytes
    64 bytes from 192.168.2.106: seq=0 ttl=64 time=0.936 ms
    64 bytes from 192.168.2.106: seq=0 ttl=64 time=1.003 ms (DUP!)
    64 bytes from 192.168.2.106: seq=1 ttl=64 time=0.802 ms
    64 bytes from 192.168.2.106: seq=2 ttl=64 time=0.696 ms
    64 bytes from 192.168.2.106: seq=3 ttl=64 time=0.664 ms
    64 bytes from 192.168.2.106: seq=4 ttl=64 time=1.194 ms
    64 bytes from 192.168.2.106: seq=5 ttl=64 time=0.613 ms
    64 bytes from 192.168.2.106: seq=6 ttl=64 time=0.898 ms
    64 bytes from 192.168.2.106: seq=8 ttl=64 time=1.066 ms
    64 bytes from 192.168.2.106: seq=9 ttl=64 time=1.144 ms
    64 bytes from 192.168.2.106: seq=9 ttl=64 time=1.219 ms (DUP!)

    --- 192.168.2.106 ping statistics ---
    10 packets transmitted, 9 packets received, 2 duplicates, 10% packet loss
    round-trip min/avg/max = 0.613/0.930/1.219 ms
    """),
    {
        "destination": "192.168.2.106",
        "packet_transmit": 10,
        "packet_receive": 9,
        "packet_duplicate_count": 2,
        "packet_duplicate_rate": 22.22222222222222,
        "packet_loss_count": 1,
        "packet_loss_rate": 9.999999999999998,
        "rtt_min": 0.613,
        "rtt_avg": 0.93,
        "rtt_max": 1.219,
        "rtt_mdev": None,
    })

WINDOWS10_LOSS = PingTestData(
    dedent("""\
    Pinging 192.168.2.106 with 32 bytes of data:
    Reply from 192.168.2.106: bytes=32 time=16ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=6ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=12ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=16ms TTL=64
    Request timed out.
    Reply from 192.168.2.106: bytes=32 time=8ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=33ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=13ms TTL=64
    Reply from 192.168.2.106: bytes=32 time=23ms TTL=64
    Reply from 192.168.2.106: bytes=32 time<1ms TTL=64

    Ping statistics for 192.168.2.106:
        Packets: Sent = 10, Received = 9, Lost = 1 (10% los
    Approximate round trip times in milli-seconds:
        Minimum = 0ms, Maximum = 33ms, Average = 14ms
    """),
    {
        "destination": "192.168.2.106",
        "packet_transmit": 10,
        "packet_receive": 9,
        "packet_loss_count": 1,
        "packet_loss_rate": 9.999999999999998,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": 0.0,
        "rtt_avg": 14.0,
        "rtt_max": 33.0,
        "rtt_mdev": None,
    })
WINDOWS_UNREACHABLE_0 = PingTestData(
    dedent("""\
    Pinging 192.168.207.100 with 32 bytes of data:
    Request timed out.
    Request timed out.
    Request timed out.
    Request timed out.

    Ping statistics for 192.168.207.100:
        Packets: Sent = 4, Received = 0, Lost = 4 (100% loss),
    """),
    {
        "destination": "192.168.207.100",
        "packet_transmit": 4,
        "packet_receive": 0,
        "packet_loss_count": 4,
        "packet_loss_rate": 100.0,
        "packet_duplicate_count": None,
        "packet_duplicate_rate": None,
        "rtt_min": None,
        "rtt_avg": None,
        "rtt_max": None,
        "rtt_mdev": None,
    })
WINDOWS_UNREACHABLE_1 = PingTestData(
    WINDOWS_UNREACHABLE_0.value + "\n", WINDOWS_UNREACHABLE_0.expected)
WINDOWS_UNREACHABLE_2 = PingTestData(
    WINDOWS_UNREACHABLE_1.value + "\n", WINDOWS_UNREACHABLE_0.expected)


class Test_PingParsing_parse(object):

    @pytest.mark.parametrize(["test_data", "parser_name"], [
        [DEBIAN_SUCCESS, "Linux"],
        [DEBIAN_UNREACHABLE_0, "Linux"],
        [DEBIAN_UNREACHABLE_1, "Linux"],
        [DEBIAN_UNREACHABLE_2, "Linux"],
        [FEDORA_DUP_LOSS, "Linux"],
        [FEDORA_UNREACHABLE, "Linux"],
        [MACOS_SUCCESS_0, "macOS"],
        [MACOS_SUCCESS_1, "macOS"],
        [MACOS_UNREACHABLE_0, "macOS"],
        [MACOS_UNREACHABLE_1, "macOS"],
        [MACOS_DUPLICATE_0, "macOS"],
        [ALPINE_LINUX_SUCCESS, "AlpineLinux"],
        [ALPINE_LINUX_DUP_LOSS, "AlpineLinux"],
        [WINDOWS7SP1_SUCCESS, "Windows"],
        [WINDOWS10_LOSS, "Windows"],
        [WINDOWS_UNREACHABLE_0, "Windows"],
        [WINDOWS_UNREACHABLE_1, "Windows"],
        [WINDOWS_UNREACHABLE_2, "Windows"],
    ])
    def test_normal_text(self, ping_parser, test_data, parser_name):
        ping_parser.parse(test_data.value)

        print("[input text]\n{}".format(test_data.value))

        assert ping_parser.parser_name == parser_name
        assert ping_parser.as_dict() == test_data.expected

    def test_empty(self, ping_parser):
        ping_parser.parse(dedent("""\
            PING google.com (216.58.196.238) 56(84) bytes of data.

            --- google.com ping statistics ---
            60 packets transmitted, 60 received, 0% packet loss, time 59153ms
            rtt min/avg/max/mdev = 61.425/99.731/212.597/27.566 ms
            """))
        ping_parser.parse("")

        assert ping_parser.destination is None
        assert ping_parser.packet_transmit is None
        assert ping_parser.packet_receive is None
        assert ping_parser.packet_loss_count is None
        assert ping_parser.packet_loss_rate is None
        assert ping_parser.packet_duplicate_count is None
        assert ping_parser.packet_duplicate_rate is None
        assert ping_parser.rtt_min is None
        assert ping_parser.rtt_avg is None
        assert ping_parser.rtt_max is None
        assert ping_parser.rtt_mdev is None

    @pytest.mark.parametrize(["value", "expected"], [
        [PING_FEDORA_EMPTY_BODY, EmptyPingStatisticsError],
        [PING_WINDOWS_INVALID, EmptyPingStatisticsError],
    ])
    def test_exception(self, ping_parser, value, expected):
        with pytest.raises(expected):
            ping_parser.parse(value)


class Test_PingParsing_as_tuple(object):
    def test_normal(self, ping_parser):
        ping_parser.parse(DEBIAN_SUCCESS.value)
        result = ping_parser.as_tuple()

        assert result.destination == "google.com"
        assert result.packet_transmit == 60
        assert result.packet_receive == 60
        assert result.packet_loss_count == 0
        assert result.packet_loss_rate == 0.0
        assert result.packet_duplicate_rate == 0
        assert result.packet_duplicate_count == 0
        assert result.rtt_min == 61.425
        assert result.rtt_avg == 99.731
        assert result.rtt_max == 212.597
        assert result.rtt_mdev == 27.566



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

from __future__ import print_function, unicode_literals

import logbook
import pytest
from pingparsing import set_log_level, set_logger


class Test_set_logger(object):

    @pytest.mark.parametrize(["value"], [
        [True],
        [False],
    ])
    def test_smoke(self, value):
        set_logger(value)


class Test_set_log_level(object):

    @pytest.mark.parametrize(["value"], [
        [logbook.CRITICAL],
        [logbook.ERROR],
        [logbook.WARNING],
        [logbook.NOTICE],
        [logbook.INFO],
        [logbook.DEBUG],
        [logbook.TRACE],
        [logbook.NOTSET],
    ])
    def test_smoke(self, value):
        set_log_level(value)

    @pytest.mark.parametrize(["value", "expected"], [
        [None, LookupError],
        ["unexpected", LookupError],
    ])
    def test_exception(self, value, expected):
        with pytest.raises(expected):
            set_log_level(value)



# encoding: utf-8

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import io
import os.path
import sys

import setuptools


MODULE_NAME = "pingparsing"
REQUIREMENT_DIR = "requirements"

pkg_info = {}


class ReleaseCommand(setuptools.Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        tag = "v{}".format(pkg_info["__version__"])

        print("Pushing git tags: {}".format(tag))

        os.system("git tag {}".format(tag))
        os.system("git push --tags")


with open(os.path.join(MODULE_NAME, "__version__.py")) as f:
    exec(f.read(), pkg_info)

with io.open("README.rst", encoding="utf8") as fp:
    long_description = fp.read()

with io.open(os.path.join(
        "docs", "pages", "introduction", "summary.txt"), encoding="utf8") as f:
    summary = f.read()

with open(os.path.join(REQUIREMENT_DIR, "requirements.txt")) as f:
    install_requires = [line.strip() for line in f if line.strip()]

with open(os.path.join(REQUIREMENT_DIR, "test_requirements.txt")) as f:
    tests_requires = [line.strip() for line in f if line.strip()]

with open(os.path.join(REQUIREMENT_DIR, "docs_requirements.txt")) as f:
    docs_requires = [line.strip() for line in f if line.strip()]

setuptools_require = ["setuptools>=20.2.2"]
needs_pytest = set(["pytest", "test", "ptr"]).intersection(sys.argv)
pytest_runner_require = ["pytest-runner"] if needs_pytest else []

setuptools.setup(
    name=MODULE_NAME,
    version=pkg_info["__version__"],
    url="https://github.com/thombashi/{:s}".format(MODULE_NAME),

    author=pkg_info["__author__"],
    author_email=pkg_info["__email__"],
    description=summary,
    include_package_data=True,
    keywords=["cli-app", "network", "ping", "parser", "transmitter"],
    license=pkg_info["__license__"],
    long_description=long_description,
    packages=setuptools.find_packages(exclude=["test*"]),

    install_requires=setuptools_require + install_requires,
    setup_requires=setuptools_require + pytest_runner_require,
    tests_require=tests_requires,
    extras_require={
        "test": tests_requires,
        "docs": docs_requires,
    },

    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Networking",
    ],
    cmdclass={
        "release": ReleaseCommand,
    },
    entry_points={
        "console_scripts": [
            "pingparsing=pingparsing.cli:main",
        ],
    })
